local _, addonTable = ...
local zc = addonTable.zc
local L = Merchant.Locales.GetString

--------------------------------------------------------------------------------

function Merchant.UI.EmptyAuctionsTable()
  for key, row in pairs(Merchant.UI.AuctionRows) do
    row.ItemTexture:SetTexture("")
    row.ItemNameText:SetText("")
    row.QuantityText:SetText("")
    row.BuyoutText:SetText("")
    row.TotalText:SetText("")
  end
  Merchant.UI.AuctionRowCnt = 1
end

--------------------------------------------------------------------------------

function Merchant.UI.AddAuctionRow(itemId, itemDisplayName, buyout, count)
  if #Merchant.UI.AuctionRows < Merchant.UI.AuctionRowCnt then
    local frame = Merchant.UI.AuctionsScrollFrameContent
    local x = 0
    local y = -10 - 35 * (Merchant.UI.AuctionRowCnt - 1)
    iconFrame = CreateFrame("Frame", nil, frame)
    iconFrame:SetSize(32, 32)
    iconFrame:SetPoint("TOPLEFT", x, y + 10)
    itemTexture = iconFrame:CreateTexture(nil, "FULLSCREEN_DIALOG")
    itemTexture:SetAllPoints()
    x = x + 32 + 10
    itemNameText = frame:CreateFontString(nil, "FULLSCREEN_DIALOG", "GameFontNormal")
    itemNameText:SetPoint("TOPLEFT", x, y)
    x = -270
    quantityText = frame:CreateFontString(nil, "FULLSCREEN_DIALOG", "GameFontNormal")
    quantityText:SetPoint("TOPRIGHT", x, y)
    x = x + 125
    buyoutText = frame:CreateFontString(nil, "FULLSCREEN_DIALOG", "GameFontNormal")
    buyoutText:SetPoint("TOPRIGHT", x, y)
    x = x + 125
    totalText = frame:CreateFontString(nil, "FULLSCREEN_DIALOG", "GameFontNormal")
    totalText:SetPoint("TOPRIGHT", x, y)
    table.insert(Merchant.UI.AuctionRows, {
      IconFrame = iconFrame,
      ItemTexture = itemTexture,
      ItemNameText = itemNameText,
      QuantityText = quantityText,
      BuyoutText = buyoutText,
      TotalText = totalText
    })
  end

  row = Merchant.UI.AuctionRows[Merchant.UI.AuctionRowCnt]
  row.ItemTexture:SetTexture(GetItemIcon(itemId))
  row.ItemNameText:SetText(itemDisplayName)
  local pixelLimit = 237
  local charLimit = 30
  local itemName = row.ItemNameText:GetText()
  local offset = 0

  local function SplitText(str, chrLimit, off, doOff)
    local leftPartoff = 0
    local rightPartoff = 0
    if doOff == true then
      if off % 2 == 0 then
         rightPartoff = 1
      else
         leftPartoff = 1
      end
    end
    local leftPart = str:sub(1, 10 + chrLimit / 2 + off + leftPartoff)
    local rightPart = str:sub(str:len() - 2 - chrLimit / 2 - off - rightPartoff, str:len())
    return leftPart, rightPart
  end

  local function ReduceText(widget, str, chrLimit, off, doOff)
    local leftPart, rightPart = SplitText(str, chrLimit, off, doOff)
    widget:SetText(leftPart .. "..." .. rightPart)
  end

  if row.ItemNameText:GetWidth() > pixelLimit then
    ReduceText(row.ItemNameText, itemName, charLimit, offset, false)
    if row.ItemNameText:GetWidth() < pixelLimit then
      local previous = row.ItemNameText:GetText()
      while true == true do
        ReduceText(row.ItemNameText, itemName, charLimit, offset, true)
        if row.ItemNameText:GetWidth() > pixelLimit then
          break
        end
        offset = offset + 1
        previous = row.ItemNameText:GetText()
      end
      row.ItemNameText:SetText(previous)
      local leftPart, rightPart = SplitText(itemName, charLimit, offset, false)
      local points = "..."
      while true == true do
        points = points .. "."
        row.ItemNameText:SetText(leftPart .. points .. rightPart)
        if row.ItemNameText:GetWidth() > pixelLimit then
          break
        end
        previous = row.ItemNameText:GetText()
      end
      row.ItemNameText:SetText(previous)
    end
  end


  row.QuantityText:SetText("|cFFFFFFFF" .. tostring(count))
  row.BuyoutText:SetText("|cFFFFFFFF" .. zc.priceToMoneyString(buyout, true))
  row.TotalText:SetText("|cFFFFFFFF" .. zc.priceToMoneyString(count * buyout, true))

  Merchant.UI.AuctionRowCnt = Merchant.UI.AuctionRowCnt + 1
  Merchant.UI.AuctionsScrollFrameContent:SetHeight(Merchant.UI.AuctionRowCnt * 34)
end

--------------------------------------------------------------------------------

function Merchant.UI.DrawAuctionsTable(charName)
  Merchant.UI.EmptyAuctionsTable()
  local auctions = Merchant_Vars.Accounts[Merchant.RealmName].Auctions[charName]
  if auctions == nil then
    Merchant.Debug("error: no auctions for " .. charName .. " character")
    return
  end
  for idx, auction in ipairs(auctions) do
    Merchant.UI.AddAuctionRow(auction.ItemId, auction.ItemDisplayName, auction.Buyout, auction.Quantity)
  end
  if #auctions > 9 then
    Merchant.UI.AuctionsScrollFrameSlider:Show()
  else
    Merchant.UI.AuctionsScrollFrameSlider:Hide()
  end
end

--------------------------------------------------------------------------------

function Merchant.UI.AuctionsDropDownHandler(self, arg1, arg2, checked)
  Merchant.UI.SelectedCharacter = arg1
  UIDropDownMenu_SetText(Merchant.UI.AuctionsDropDown, Merchant.Utils.GetPrettyCharacterName(arg1))
  Merchant.UI.DrawAuctionsTable(arg1)
end

--------------------------------------------------------------------------------

function Merchant.UI.TestAuctionsOrder(label, compareFunc)
  local auctions = Merchant_Vars.Accounts[Merchant.RealmName].Auctions[Merchant.UI.SelectedCharacter]
  if #auctions > 1 then
    local previous = auctions[1][label]
    for i = 2, #auctions do
      local current = auctions[i][label]
      if compareFunc(previous, current) == false then
        return false
      end
      previous = current
    end
  end
  return true
end

--------------------------------------------------------------------------------

Merchant.UI.SortFuncs = {
  ItemNameAsc = function(a, b)
    return a.ItemName:upper() < b.ItemName:upper()
  end,
  ItemNameDesc = function(a, b)
    return a.ItemName:upper() > b.ItemName:upper()
  end,
  QuantityAsc = function(a, b)
    return a.Quantity < b.Quantity
  end,
  QuantityDesc = function(a, b)
    return a.Quantity > b.Quantity
  end,
  BuyoutAsc = function(a, b)
    return a.Buyout < b.Buyout
  end,
  BuyoutDesc = function(a, b)
    return a.Buyout > b.Buyout
  end,
  TotalAsc = function(a, b)
    return a.Total < b.Total
  end,
  TotalDesc = function(a, b)
    return a.Total > b.Total
  end
}

--------------------------------------------------------------------------------

function Merchant.UI.SortAuctions(frame)
  local test = false
  if Merchant.UI.SortBy == nil then
    Merchant.UI.SortBy = frame.Label
    Merchant.UI.SortOrder = "Asc"
    test = true
  elseif Merchant.UI.SortBy == frame.Label then
    if Merchant.UI.SortOrder == "Desc" then
      Merchant.UI.SortOrder = "Asc"
    else
      Merchant.UI.SortOrder = "Desc"
    end
  else
    Merchant.UI.SortBy = frame.Label
    Merchant.UI.SortOrder = "Asc"
    test = true
  end

  local doSort = true
  if test == true then
    local areAsc = Merchant.UI.TestAuctionsOrder(Merchant.UI.SortBy,
      function(a, b) return a <= b end)
    local areDesc = Merchant.UI.TestAuctionsOrder(Merchant.UI.SortBy,
      function(a, b) return a >= b end)

    if areAsc == true then
      Merchant.UI.SortOrder = "Desc"
    end

    if areDesc == true then
      Merchant.UI.SortOrder = "Asc"
    end

    if areAsc == true and areDesc == true then
      doSort = false
    end
  end

  if doSort == true then
    table.sort(Merchant_Vars.Accounts[Merchant.RealmName].Auctions[Merchant.UI.SelectedCharacter], Merchant.UI.SortFuncs[Merchant.UI.SortBy .. Merchant.UI.SortOrder])
    Merchant.UI.DrawAuctionsTable(Merchant.UI.SelectedCharacter)
  end
end

--------------------------------------------------------------------------------

function Merchant.UI.FillAuctionsFrame(tabFrame)
  Merchant.UI.AuctionsFrame = tabFrame
  Merchant.UI.AuctionsScrollFrame = CreateFrame("ScrollFrame", "$parentScrollFrame", Merchant.UI.AuctionsFrame)
  Merchant.UI.AuctionsScrollFrame:EnableMouseWheel(true)
  Merchant.UI.AuctionsScrollFrame:SetPoint("TOPLEFT", 0, -42 - 20 - 10)
  Merchant.UI.AuctionsScrollFrame:SetPoint("BOTTOMRIGHT", 0, 5)
  Merchant.UI.AuctionsScrollFrame:SetScript("OnMouseWheel",
    function(frame, value)
      Merchant.UI.AuctionsScrollFrameSlider:SetValue(math.min(math.max(Merchant.UI.AuctionsScrollValue + value * (1000 / ((Merchant.UI.AuctionsScrollFrame:GetHeight() - Merchant.UI.AuctionsScrollFrameContent:GetHeight()) / 45)), 0), 1000))
  end)
  Merchant.UI.AuctionsScrollFrameSlider = CreateFrame("Slider", "$parentSlider", Merchant.UI.AuctionsScrollFrame, "UIPanelScrollBarTemplate")
  Merchant.UI.AuctionsScrollFrameSlider:SetPoint("TOPLEFT", Merchant.UI.AuctionsScrollFrame, "TOPRIGHT", -5, 0)
  Merchant.UI.AuctionsScrollFrameSlider:SetPoint("BOTTOMLEFT", Merchant.UI.AuctionsScrollFrame, "BOTTOMRIGHT", 0, 0)
  Merchant.UI.AuctionsScrollFrameSlider:SetMinMaxValues(0, 1000)
  Merchant.UI.AuctionsScrollFrameSlider:SetValueStep(1)
  Merchant.UI.AuctionsScrollFrameSlider:SetValue(0)
  Merchant.UI.AuctionsScrollFrameSlider:SetWidth(16)

  local function scrollRefresh(value)
    local scrollFrameHeight = Merchant.UI.AuctionsScrollFrame:GetHeight()
    local contentHeight = Merchant.UI.AuctionsScrollFrameContent:GetHeight()
    local offset
    if scrollFrameHeight > contentHeight then
      offset = 0
    else
      offset = math.floor((contentHeight - scrollFrameHeight) / 1000.0 * value)
      offset = offset - offset % 35
    end
    Merchant.UI.AuctionsScrollFrameContent:ClearAllPoints()
    Merchant.UI.AuctionsScrollFrameContent:SetPoint("TOPLEFT", 0, offset)
    Merchant.UI.AuctionsScrollFrameContent:SetPoint("TOPRIGHT", 0, offset)
    Merchant.UI.AuctionsScrollValue = value
  end

  Merchant.UI.AuctionsScrollFrameSlider:SetScript("OnValueChanged", function(frame, value)
      scrollRefresh(value)
  end)

  Merchant.UI.AuctionsScrollFrameSliderBackground = Merchant.UI.AuctionsScrollFrameSlider:CreateTexture(nil, "BACKGROUND")
  Merchant.UI.AuctionsScrollFrameSliderBackground:SetAllPoints(Merchant.UI.AuctionsScrollFrameSlider)
  Merchant.UI.AuctionsScrollFrameSliderBackground:SetColorTexture(0, 0, 0, 0.4)

  Merchant.UI.AuctionsScrollFrameContent = CreateFrame("Frame", "$parentContent", Merchant.UI.AuctionsScrollFrame)
  Merchant.UI.AuctionsScrollFrameContent:SetPoint("TOPLEFT")
  Merchant.UI.AuctionsScrollFrameContent:SetPoint("TOPRIGHT")
  Merchant.UI.AuctionsScrollFrameContent:SetHeight(0)

  Merchant.UI.AuctionsScrollFrame:SetScrollChild(Merchant.UI.AuctionsScrollFrameContent)


  local fixedContent = Merchant.UI.AuctionsFrame
  Merchant.UI.AuctionsDropDown = CreateFrame("Frame", "$parentDropDown", fixedContent, "UIDropDownMenuTemplate")
  Merchant.UI.AuctionsDropDown:ClearAllPoints()
  Merchant.UI.AuctionsDropDown:SetPoint("TOPLEFT", -20, 0)
  UIDropDownMenu_SetWidth(Merchant.UI.AuctionsDropDown, 125)
  UIDropDownMenu_Initialize(Merchant.UI.AuctionsDropDown, function(frame, level, menuList)
    local info = UIDropDownMenu_CreateInfo()
    info.notCheckable = true
    info.func = Merchant.UI.AuctionsDropDownHandler
    for key, value
      in pairs(Merchant_Vars.Accounts[Merchant.RealmName].Auctions) do
      info.arg1 = key
      info.text = Merchant.Utils.GetPrettyCharacterName(key) .. "                  "
      UIDropDownMenu_AddButton(info)
      default = key
    end
  end)
  for key, value in
    pairs(Merchant_Vars.Accounts[Merchant.RealmName].Auctions) do
    if key == Merchant.CharName then
      Merchant.UI.SelectedCharacter = key
    end
  end
  if Merchant.UI.SelectedCharacter == nil then
    Merchant.UI.SelectedCharacter = default
  end
  UIDropDownMenu_SetText(Merchant.UI.AuctionsDropDown, Merchant.Utils.GetPrettyCharacterName(Merchant.UI.SelectedCharacter))

  local template = "OptionsFrameTabButtonTemplate"
  local x = 30
  local y = -42
  nameButton = CreateFrame("Button", "$parentNameButton", fixedContent, template)
  nameButton:SetPoint("TOPLEFT", x, y)
  local str = ""
  for i = 1, 28 do
    str = str .. " "
  end
  str = str .. "|cFF" .. Merchant.Constants.UI.SECOND_COLOR .. L("TITLE_NAME")
  for i = 1, 28 do
    str = str .. " "
  end
  nameButton:SetText(str)
  nameButton.Label = "ItemName"
  nameButton:SetScript("OnClick", Merchant.UI.SortAuctions)
  x = -235
  quantityButton = CreateFrame("Button", "$parentQuantityButton", fixedContent, template)
  quantityButton:SetPoint("TOPRIGHT", x, y)
  quantityButton:SetText("|cFF" .. Merchant.Constants.UI.SECOND_COLOR .. L("TITLE_QUANTITY"))
  quantityButton.Label = "Quantity"
  quantityButton:SetScript("OnClick", Merchant.UI.SortAuctions)
  x = x + 97
  buyoutButton = CreateFrame("Button", "$parentBuyoutButton", fixedContent, template)
  buyoutButton:SetPoint("TOPRIGHT", x, y)
  buyoutButton:SetText("  |cFF" .. Merchant.Constants.UI.SECOND_COLOR .. L("TITLE_BUYOUT") .. "  ")
  buyoutButton.Label = "Buyout"
  buyoutButton:SetScript("OnClick", Merchant.UI.SortAuctions)
  x = x + 123
  totalButton = CreateFrame("Button", "$parentTotalButton", fixedContent, template)
  totalButton:SetPoint("TOPRIGHT", x, y)
  totalButton:SetText("     |cFF" .. Merchant.Constants.UI.SECOND_COLOR .. L("TITLE_TOTAL") .. "     ")
  totalButton.Label = "Total"
  totalButton:SetScript("OnClick", Merchant.UI.SortAuctions)

  scrollRefresh(0)
end
