local _, addonTable = ...
local zc = addonTable.zc
local L = Merchant.Locales.GetString

--------------------------------------------------------------------------------

function Merchant.UI.StockpileCharsDropDownHandler(self, arg1, arg2, checked)
  Merchant.UI.StockpileSelectedCharacter = arg1
  local pretty = Merchant.Utils.GetPrettyCharacterName(Merchant.UI.StockpileSelectedCharacter)
  UIDropDownMenu_SetText(Merchant.UI.StockpileCharsDropDown, pretty)
  Merchant.UI.DrawStockpile()
end

--------------------------------------------------------------------------------

function Merchant.UI.CreateCharsDropdown(fixedContent)
  Merchant.UI.StockpileCharsDropDown = CreateFrame("Frame", "$parentCharsDropDown", fixedContent, "UIDropDownMenuTemplate")
  Merchant.UI.StockpileCharsDropDown:ClearAllPoints()
  Merchant.UI.StockpileCharsDropDown:SetPoint("TOPLEFT", -20, 0)
  UIDropDownMenu_SetWidth(Merchant.UI.StockpileCharsDropDown, 125)
  UIDropDownMenu_Initialize(Merchant.UI.StockpileCharsDropDown, function(frame, level, menuList)
    local info = UIDropDownMenu_CreateInfo()
    info.notCheckable = true
    info.func = Merchant.UI.StockpileCharsDropDownHandler
    for key, value in pairs(Merchant_Vars.Accounts[Merchant.RealmName].Stockpile) do
      info.arg1 = key
      info.text = Merchant.Utils.GetPrettyCharacterName(key) .. "                  "
      UIDropDownMenu_AddButton(info)
      default = key
    end
  end)
  for key, value in pairs(Merchant_Vars.Accounts[Merchant.RealmName].Stockpile) do
    if key == Merchant.CharName then
      Merchant.UI.StockpileSelectedCharacter = key
    end
  end
  if Merchant.UI.StockpileSelectedCharacter == nil then
    Merchant.UI.StockpileSelectedCharacter = default
  end
  UIDropDownMenu_SetText(Merchant.UI.StockpileCharsDropDown, Merchant.Utils.GetPrettyCharacterName(Merchant.UI.StockpileSelectedCharacter))
end

--------------------------------------------------------------------------------

function Merchant.UI.StockpileDisplayDropDownHandler(self, arg1, arg2, checked)
  Merchant.UI.StockpileSelectedDisplay = arg1
  local pretty = nil
  if Merchant.UI.StockpileSelectedDisplay == "grid" then
    pretty = "Grid View"
  else
    pretty = "Table View"
  end
  UIDropDownMenu_SetText(Merchant.UI.StockpileDisplayDropDown, pretty)
  Merchant.UI.DrawStockpile()
end

--------------------------------------------------------------------------------

function Merchant.UI.CreateDisplayDropdown(fixedContent)
  Merchant.UI.StockpileDisplayDropDown = CreateFrame("Frame", "$parentDisplayDropDown", fixedContent, "UIDropDownMenuTemplate")
  Merchant.UI.StockpileDisplayDropDown:ClearAllPoints()
  Merchant.UI.StockpileDisplayDropDown:SetPoint("TOPRIGHT", 0, 0)
  UIDropDownMenu_SetWidth(Merchant.UI.StockpileDisplayDropDown, 125)
  UIDropDownMenu_Initialize(Merchant.UI.StockpileDisplayDropDown, function(frame, level, menuList)
    local info = UIDropDownMenu_CreateInfo()
    info.notCheckable = true
    info.func = Merchant.UI.StockpileDisplayDropDownHandler
    info.arg1 = "grid"
    info.text = "Grid View"
    UIDropDownMenu_AddButton(info)
    info.arg1 = "table"
    info.text = "Table View"
    UIDropDownMenu_AddButton(info)
  end)
  Merchant.UI.StockpileSelectedDisplay = "table"
  UIDropDownMenu_SetText(Merchant.UI.StockpileDisplayDropDown, "Table View")
end

--------------------------------------------------------------------------------

function Merchant.UI.StockpileFiltersDropDownHandler(self, arg1, arg2, checked)
  Merchant.UI.StockpileSelectedFilter = arg1
  if Merchant.UI.StockpileSelectedFilter == "all" then
    pretty = L("ALL")
  else
    pretty = L(Merchant.Constants.UI.TRADESKILL_KEYS[Merchant.UI.StockpileSelectedFilter])
  end
  UIDropDownMenu_SetText(Merchant.UI.StockpileFiltersDropDown, pretty)
  Merchant.UI.DrawStockpile()
end

--------------------------------------------------------------------------------

function Merchant.UI.CreateFiltersDropdown(fixedContent)
  Merchant.UI.StockpileFiltersDropDown = CreateFrame("Frame", "$parentDropDown", fixedContent, "UIDropDownMenuTemplate")
  Merchant.UI.StockpileFiltersDropDown:ClearAllPoints()
  Merchant.UI.StockpileFiltersDropDown:SetPoint("TOPLEFT", 220, 0)
  UIDropDownMenu_SetWidth(Merchant.UI.StockpileFiltersDropDown, 125)
  UIDropDownMenu_Initialize(Merchant.UI.StockpileFiltersDropDown, function(frame, level, menuList)
    local info = UIDropDownMenu_CreateInfo()
    info.notCheckable = true
    info.func = Merchant.UI.StockpileFiltersDropDownHandler
    info.arg1 = "all"
    info.text = L("ALL")
    UIDropDownMenu_AddButton(info)
    local check = {}
    for key, value in pairs(Merchant_Vars.Accounts[Merchant.RealmName].Stockpile[Merchant.UI.StockpileSelectedCharacter]) do
      if check[value.ItemSubClassID] == nil then
        info.arg1 = value.ItemSubClassID
        info.text = L(Merchant.Constants.UI.TRADESKILL_KEYS[value.ItemSubClassID])
        UIDropDownMenu_AddButton(info)
        check[value.ItemSubClassID] = true
      end
    end
  end)
  Merchant.UI.StockpileSelectedFilter = "all"
  UIDropDownMenu_SetText(Merchant.UI.StockpileFiltersDropDown, L("ALL"))
end

--------------------------------------------------------------------------------

function Merchant.UI.DrawStockpile()
  if Merchant.UI.StockpileSelectedDisplay == "grid" then
    Merchant.UI.StockpileTableFrame:Hide()
    Merchant.UI.StockpileGridFrame:Show()
    Merchant.UI.DrawStockpileGrid(Merchant.UI.StockpileSelectedCharacter, Merchant.UI.StockpileSelectedFilter)
  else
    Merchant.UI.StockpileTableFrame:Show()
    Merchant.UI.StockpileGridFrame:Hide()
    Merchant.UI.DrawStockpileTable(Merchant.UI.StockpileSelectedCharacter, Merchant.UI.StockpileSelectedFilter)
  end
end

--------------------------------------------------------------------------------

function Merchant.UI.FillStockpileFrame(tabFrame)
  Merchant.UI.StockpileFrame = tabFrame

  -- Grid
  Merchant.UI.StockpileGridFrame = CreateFrame("Frame", "$parentGridFrame", Merchant.UI.StockpileFrame)
  Merchant.UI.StockpileGridFrame:SetAllPoints()
  Merchant.UI.StockpileGridScrollFrame = CreateFrame("ScrollFrame", "$parentGridScrollFrame", Merchant.UI.StockpileGridFrame)
  Merchant.UI.StockpileGridScrollFrame:EnableMouseWheel(true)
  Merchant.UI.StockpileGridScrollFrame:SetPoint("TOPLEFT", 0, -42)
  Merchant.UI.StockpileGridScrollFrame:SetPoint("BOTTOMRIGHT", 0, 5)
  Merchant.UI.StockpileGridScrollFrame:SetScript("OnMouseWheel", function(frame, value)
    Merchant.UI.StockpileGridScrollFrameSlider:SetValue(math.min(math.max(Merchant.UI.StockpileGridScrollValue + value * (1000 / ((Merchant.UI.StockpileGridScrollFrame:GetHeight() - Merchant.UI.StockpileGridScrollFrameContent:GetHeight()) / 45)), 0), 1000))
  end)
  Merchant.UI.StockpileGridScrollFrameSlider = CreateFrame("Slider", "$parentSlider", Merchant.UI.StockpileGridScrollFrame, "UIPanelScrollBarTemplate")
  Merchant.UI.StockpileGridScrollFrameSlider:SetPoint("TOPLEFT", Merchant.UI.StockpileGridScrollFrame, "TOPRIGHT", -5, 0)
  Merchant.UI.StockpileGridScrollFrameSlider:SetPoint("BOTTOMLEFT", Merchant.UI.StockpileGridScrollFrame, "BOTTOMRIGHT", 0, 0)
  Merchant.UI.StockpileGridScrollFrameSlider:SetMinMaxValues(0, 1000)
  Merchant.UI.StockpileGridScrollFrameSlider:SetValueStep(1)
  Merchant.UI.StockpileGridScrollFrameSlider:SetValue(0)
  Merchant.UI.StockpileGridScrollFrameSlider:SetWidth(16)
  Merchant.UI.StockpileGridScrollFrameSlider:SetScript("OnValueChanged", function(frame, value)
    Merchant.UI.StockpileGridScrollRefresh(value)
  end)
  Merchant.UI.StockpileGridScrollFrameSliderBackground = Merchant.UI.StockpileGridScrollFrameSlider:CreateTexture(nil, "BACKGROUND")
  Merchant.UI.StockpileGridScrollFrameSliderBackground:SetAllPoints(Merchant.UI.StockpileGridScrollFrameSlider)
  Merchant.UI.StockpileGridScrollFrameSliderBackground:SetColorTexture(0, 0, 0, 0.4)
  Merchant.UI.StockpileGridScrollFrameContent = CreateFrame("Frame", "$parentContent", Merchant.UI.StockpileGridScrollFrame)
  Merchant.UI.StockpileGridScrollFrameContent:SetPoint("TOPLEFT")
  Merchant.UI.StockpileGridScrollFrameContent:SetPoint("TOPRIGHT")
  Merchant.UI.StockpileGridScrollFrameContent:SetHeight(0)
  Merchant.UI.StockpileGridScrollFrame:SetScrollChild(Merchant.UI.StockpileGridScrollFrameContent)

  -- Table
  Merchant.UI.StockpileTableFrame = CreateFrame("Frame", "$parentTableFrame", Merchant.UI.StockpileFrame)
  Merchant.UI.StockpileTableFrame:SetAllPoints()
  Merchant.UI.StockpileTableScrollFrame = CreateFrame("ScrollFrame", "$parentScrollFrame", Merchant.UI.StockpileTableFrame)
  Merchant.UI.StockpileTableScrollFrame:EnableMouseWheel(true)
  Merchant.UI.StockpileTableScrollFrame:SetPoint("TOPLEFT", 0, -42 - 20 - 10)
  Merchant.UI.StockpileTableScrollFrame:SetPoint("BOTTOMRIGHT", 0, 5)
  Merchant.UI.StockpileTableScrollFrame:SetScript("OnMouseWheel", function(frame, value)
    Merchant.UI.StockpileTableScrollFrameSlider:SetValue(math.min(math.max(Merchant.UI.StockpileTableScrollValue + value * (1000 / ((Merchant.UI.StockpileTableScrollFrame:GetHeight() - Merchant.UI.StockpileTableScrollFrameContent:GetHeight()) / 45)), 0), 1000))
  end)
  Merchant.UI.StockpileTableScrollFrameSlider = CreateFrame("Slider", "$parentSlider", Merchant.UI.StockpileTableScrollFrame, "UIPanelScrollBarTemplate")
  Merchant.UI.StockpileTableScrollFrameSlider:SetPoint("TOPLEFT", Merchant.UI.StockpileTableScrollFrame, "TOPRIGHT", -5, 0)
  Merchant.UI.StockpileTableScrollFrameSlider:SetPoint("BOTTOMLEFT", Merchant.UI.StockpileTableScrollFrame, "BOTTOMRIGHT", 0, 0)
  Merchant.UI.StockpileTableScrollFrameSlider:SetMinMaxValues(0, 1000)
  Merchant.UI.StockpileTableScrollFrameSlider:SetValueStep(1)
  Merchant.UI.StockpileTableScrollFrameSlider:SetValue(0)
  Merchant.UI.StockpileTableScrollFrameSlider:SetWidth(16)
  Merchant.UI.StockpileTableScrollFrameSlider:SetScript("OnValueChanged", function(frame, value)
    Merchant.UI.StockpileTableScrollRefresh(value)
  end)
  Merchant.UI.StockpileTableScrollFrameSliderBackground = Merchant.UI.StockpileTableScrollFrameSlider:CreateTexture(nil, "BACKGROUND")
  Merchant.UI.StockpileTableScrollFrameSliderBackground:SetAllPoints(Merchant.UI.StockpileTableScrollFrameSlider)
  Merchant.UI.StockpileTableScrollFrameSliderBackground:SetColorTexture(0, 0, 0, 0.4)
  Merchant.UI.StockpileTableScrollFrameContent = CreateFrame("Frame", "$parentContent", Merchant.UI.StockpileTableScrollFrame)
  Merchant.UI.StockpileTableScrollFrameContent:SetPoint("TOPLEFT")
  Merchant.UI.StockpileTableScrollFrameContent:SetPoint("TOPRIGHT")
  Merchant.UI.StockpileTableScrollFrameContent:SetHeight(0)
  Merchant.UI.StockpileTableScrollFrame:SetScrollChild(Merchant.UI.StockpileTableScrollFrameContent)

  local fixedContent = Merchant.UI.StockpileTableFrame

  local template = "OptionsFrameTabButtonTemplate"
  local x = 30
  local y = -42
  nameButton = CreateFrame("Button", "$parentNameButton", fixedContent, template)
  nameButton:SetPoint("TOPLEFT", x, y)
  local str = ""
  for i = 1, 28 do
    str = str .. " "
  end
  str = str .. "|cFF" .. Merchant.Constants.UI.SECOND_COLOR .. L("TITLE_NAME")
  for i = 1, 28 do
    str = str .. " "
  end
  nameButton:SetText(str)
  nameButton.Label = "ItemName"
  nameButton:SetScript("OnClick", Merchant.UI.SortStockpile)
  x = -235
  quantityButton = CreateFrame("Button", "$parentQuantityButton", fixedContent, template)
  quantityButton:SetPoint("TOPRIGHT", x, y)
  quantityButton:SetText("|cFF" .. Merchant.Constants.UI.SECOND_COLOR .. L("TITLE_QUANTITY"))
  quantityButton.Label = "ItemCount"
  quantityButton:SetScript("OnClick", Merchant.UI.SortStockpile)
  x = x + 97
  buyoutButton = CreateFrame("Button", "$parentBuyoutButton", fixedContent, template)
  buyoutButton:SetPoint("TOPRIGHT", x, y)
  buyoutButton:SetText("  |cFF" .. Merchant.Constants.UI.SECOND_COLOR .. L("TITLE_VALUE") .. "  ")
  buyoutButton.Label = "Value"
  buyoutButton:SetScript("OnClick", Merchant.UI.SortStockpile)
  x = x + 123
  totalButton = CreateFrame("Button", "$parentTotalButton", fixedContent, template)
  totalButton:SetPoint("TOPRIGHT", x, y)
  totalButton:SetText("     |cFF" .. Merchant.Constants.UI.SECOND_COLOR .. L("TITLE_TOTAL") .. "     ")
  totalButton.Label = "Total"
  totalButton:SetScript("OnClick", Merchant.UI.SortStockpile)


  Merchant.UI.CreateCharsDropdown(Merchant.UI.StockpileFrame)
  Merchant.UI.CreateDisplayDropdown(Merchant.UI.StockpileFrame)
  Merchant.UI.CreateFiltersDropdown(Merchant.UI.StockpileFrame)

  Merchant.UI.StockpileSelectedDisplay = "table"

  Merchant.UI.StockpileGridScrollRefresh(0)
  Merchant.UI.StockpileTableScrollRefresh(0)

  Merchant.UI.StockpileGridFrame:Hide()
end
